@@ -5,23 +5,9 @@
 
 	Designed and implemented in July of 1991 by John Walker
 
-    Release 5: August 1995.  Added GSM compression, DES, IDEA, and
-			     key file encryption.
-
-    Release 5.1: September 1995.  Added ADPCM compression, made
-			     push-to-talk mode the default, switched
-			     to smaller packets to avoid fragging the
-			     datagram sender, optimised utilisation of
-			     the available packet length.
-
-    Release 5.2: September 1995.  New hat!!!!  Renamed to "Speak Freely
-                             for Unix".  SGI packet length made compatible
-			     with Windows version.  Platform-dependent
-			     soundflush() function introduced.
-
 */
 
-#include "netfone.h"
+#include "speakfree.h"
 
 /*  Destination host descriptor.  */
 
@@ -29,6 +15,10 @@ struct destination {
     struct destination *dnext;	      /* Next destination in list */
     char *server;		      /* Host name identifier string */
     struct sockaddr_in name;	      /* Internet address */
+    struct sockaddr_in ctrl;	      /* RTCP port address */
+#ifdef RTP_SUPPORT
+    int isrtp;			      /* Is this an RTP connection ? */
+#endif
     unsigned char time_to_live;       /* Multicast scope specification */
     char deskey[9];		      /* Destination DES key, if any */
     char ideakey[17];		      /* Destination IDEA key, if any */
@@ -40,7 +30,8 @@ static char *progname;		      /* Program name */
 static int sock;		      /* Communication socket */
 static struct destination *dests = NULL, *dtail;
 static int compressing = FALSE;       /* Compress sound: simple 2X */
-static int gsmcompress = FALSE;       /* GSM compress buffers */
+static int gsmcompress = TRUE;	      /* GSM compress buffers */
+static int lpccompress = FALSE;       /* LPC compress buffers */
 static int adpcmcompress = FALSE;     /* ADPCM compress buffers */
 static int squelch = 0; 	      /* Squelch level if > 0 */
 #ifdef PUSH_TO_TALK
@@ -49,6 +40,9 @@ static int talking = FALSE;	      /* Push to talk button state */
 static int rawmode = FALSE;	      /* Is terminal in raw mode ? */
 #endif
 static int ring = FALSE;	      /* Force speaker & level on next pkt ? */
+#ifdef RTP_SUPPORT
+static int rtp = FALSE; 	      /* Use Internet Real-Time Protocol */
+#endif
 static int agc = FALSE; 	      /* Automatic gain control active ? */
 static int rgain = 33;		      /* Current recording gain level */
 static int debugging = FALSE;	      /* Debugging enabled here and there ? */
@@ -57,6 +51,14 @@ static char hostname[20];	      /* Host name to send with packets */
 static int loopback = FALSE;	      /* Remote loopback mode */
 static gsm gsmh;		      /* GSM handle */
 static struct adpcm_state adpcm;      /* ADPCM compression state */
+
+static unsigned long ssrc;	      /* RTP synchronisation source identifier */
+static unsigned long timestamp;       /* RTP packet timestamp */
+static unsigned short seq;	      /* RTP packet sequence number */
+
+static char *sdes = NULL;	      /* RTP SDES packet */
+static int sdesl;		      /* RTP SDES packet length */
+
 static char curkey[9] = "";           /* Current DES key if curkey[0] != 0 */
 static char curideakey[17] = "";      /* Current IDEA key if curideakey[0] != 0 */
 static char curpgpkey[17] = "";       /* Current PGP key if curpgpkey[0] != 0 */
@@ -65,15 +67,19 @@ static char *curotp = NULL;	      /* Key file buffer */
 static int sound_packet;	      /* Ideal samples/packet */
 #endif
 static struct soundbuf *pgpsb = NULL; /* PGP key sound buffer, if any */
+static LONG pgpsbl;		      /* Length of PGP key sound buffer data */
 #ifdef HALF_DUPLEX
 static struct in_addr localhost;      /* Our internet address */
 static int halfDuplexMuted = FALSE;   /* Muted by half-duplex transmission */
 #endif
+static int hasFace = FALSE;	      /* Is a face image available ? */
 
 #ifdef sgi
 static long usticks;		      /* Microseconds per clock tick */
 #endif
 
+#define ucase(x)    (islower(x) ? toupper(x) : (x))
+
 /*  GSMCOMP  --  Compress the contents of a sound buffer using GSM.  */
 
 static void gsmcomp(sb)
@@ -129,6 +135,30 @@ static void adpcmcomp(sb)
     sb->buffer.buffer_len += 3;
 }
 
+/*  LPCCOMP  --  Compress the contents of a sound buffer using LPC.  */
+
+static void lpccomp(sb)
+  struct soundbuf *sb;
+{
+    int i, l = 0;
+    char *dp = ((char *) sb->buffer.buffer_val) + sizeof(short);
+    unsigned char *src = ((unsigned char *) sb->buffer.buffer_val);
+    lpcparams_t lp;
+
+    sb->compression |= fCompLPC;
+    for (i = 0; i < sb->buffer.buffer_len; i += 180) {
+	lpc_analyze(src + i, &lp);
+	bcopy(&lp, dp, sizeof lp);
+	dp += sizeof lp;
+	l += sizeof lp;
+    }
+
+    /* Hide original uncompressed buffer length in first 2 bytes of buffer. */
+
+    *((short *) sb->buffer.buffer_val) = sb->buffer.buffer_len;
+    sb->buffer.buffer_len = l + sizeof(short);
+}
+
 /*  ADDEST  --	Add destination host to host list.  */
 
 static int addest(host)
@@ -139,6 +169,7 @@ static int addest(host)
     long naddr;
     unsigned int ttl = 1;
     char *mcs;
+    int curport = Internet_Port;
 
     /* If a multicast scope descriptor appears in the name, scan
        it and lop it off the end.  We'll apply it later if we discover
@@ -151,6 +182,14 @@ static int addest(host)
 #endif
     }
 
+    /* If a port number appears in the name, scan it and lop
+       it off. */
+
+    if ((mcs = strchr(host, ':')) != NULL) {
+	*mcs++ = 0;
+	curport = atoi(mcs);
+    }
+
     /* If it's a valid IP number, use it.  Otherwise try to look
        up as a host name. */
 
@@ -174,13 +213,19 @@ static int addest(host)
     d->dnext = NULL;
     d->server = host;
     bcopy((char *) &naddr, (char *) &(d->name.sin_addr), sizeof naddr);
+    bcopy((char *) &naddr, (char *) &(d->ctrl.sin_addr), sizeof naddr);
     bcopy(curkey, d->deskey, 9);
     bcopy(curideakey, d->ideakey, 17);
     bcopy(curpgpkey, d->pgpkey, 17);
     d->otp = curotp;
     d->time_to_live = ttl;
+#ifdef RTP_SUPPORT
+    d->isrtp = rtp;
+#endif
     d->name.sin_family = AF_INET;
-    d->name.sin_port = htons(Internet_Port);
+    d->name.sin_port = htons(curport);
+    d->ctrl.sin_family = AF_INET;
+    d->ctrl.sin_port = htons(curport + 1);
     if (dests == NULL) {
 	dests = d;
     } else {
@@ -188,6 +233,16 @@ static int addest(host)
     }
     dtail = d;
 
+#ifdef RTP_SUPPORT
+    if (d->isrtp) {
+	if (sendto(sock, sdes, sdesl,
+	     0, (struct sockaddr *) &(d->ctrl), sizeof d->ctrl) < 0) {
+            perror("sending RTCP packet");
+	    return FALSE;
+	}
+    }
+#endif
+
     if (pgpsb != NULL) {
 	int i;
 
@@ -197,8 +252,7 @@ static int addest(host)
 	   arrived safely. */
 
 	for (i = 0; i < 3; i++) {
-	    if (sendto(sock, pgpsb, (sizeof(struct soundbuf) - BUFL) +
-				    pgpsb->buffer.buffer_len,
+	    if (sendto(sock, (char *) pgpsb, (sizeof(struct soundbuf) - BUFL) + pgpsbl,
 		0, (struct sockaddr *) &(d->name), sizeof d->name) < 0) {
                 perror("sending PGP session key");
 		break;
@@ -225,6 +279,14 @@ static int sendpkt(sb)
 	adpcmcomp(sb);
     }
 
+    if (lpccompress) {
+	lpccomp(sb);
+    }
+
+    if (hasFace) {
+	sb->compression |= fFaceOffer;
+    }
+
     for (d = dests; d != NULL; d = d->dnext) {
 #ifdef MULTICAST
 	if (IN_MULTICAST(d->name.sin_addr.s_addr)) {
@@ -331,14 +393,38 @@ static int sendpkt(sb)
 		}
 		ebuf.compression |= fEncOTP;
 	    }
-	    if (sendto(sock, &ebuf, (sizeof(struct soundbuf) - BUFL) + slen,
+
+	    ebuf.compression = htonl(ebuf.compression);
+	    ebuf.buffer.buffer_len = htonl(ebuf.buffer.buffer_len);
+	    if (sendto(sock, (char *) &ebuf, (sizeof(struct soundbuf) - BUFL) + slen,
 		0, (struct sockaddr *) &(d->name), sizeof d->name) < 0) {
                 perror("sending datagram message");
 		return FALSE;
 	    }
 	} else {
-	    if (sendto(sock, sb, (sizeof(struct soundbuf) - BUFL) +
-				    sb->buffer.buffer_len,
+	    int pktlen = (int) ((sizeof(struct soundbuf) - BUFL) +
+				    sb->buffer.buffer_len);
+
+#ifdef RTP_SUPPORT
+	    if (d->isrtp) {
+		LONG bufl = sb->buffer.buffer_len;
+
+		if (sendto(sock, sdes, sdesl,
+		    0, (struct sockaddr *) &(d->ctrl), sizeof d->ctrl) < 0) {
+                    perror("sending RTCP packet");
+		    return FALSE;
+		}
+		pktlen = rtpout(sb, ssrc, timestamp, seq);
+		seq++;
+		timestamp += sound_packet;
+	    } else {
+#endif
+		sb->compression = htonl(sb->compression);
+		sb->buffer.buffer_len = htonl(sb->buffer.buffer_len);
+#ifdef RTP_SUPPORT
+	    }
+#endif
+	    if (sendto(sock, (char *) sb, pktlen,
 		0, (struct sockaddr *) &(d->name), sizeof d->name) < 0) {
                 perror("sending datagram message");
 		return FALSE;
@@ -366,20 +452,22 @@ static int getaudio()
        fails, he'll know it was a result of being muted. */
 
     for (i = 5; i > 0; i--) {
-	hdreq.compression = fHalfDuplex | fHalfDuplexMute;
+	hdreq.compression = htonl(fHalfDuplex | fHalfDuplexMute);
 	strcpy(hdreq.sendinghost, hostname);
 	hdreq.buffer.buffer_len = 0;
 
 	bcopy((char *) &localhost, (char *) &(name.sin_addr), sizeof localhost);
 	name.sin_family = AF_INET;
 	name.sin_port = htons(Internet_Port);
-	if (sendto(sock, &hdreq, (sizeof(struct soundbuf) - BUFL) +
-		   hdreq.buffer.buffer_len,
+	if (debugging) {
+            fprintf(stderr,"Sending fHalfDuplex | fHalfDuplexMute Request\n");
+	}
+	if (sendto(sock, &hdreq, (sizeof(struct soundbuf) - BUFL),
 		   0, (struct sockaddr *) &name, sizeof name) < 0) {
             perror("sending half-duplex mute request");
 	    break;
 	}
-#ifdef BSD_like
+#if defined(BSD_like) || defined(Solaris)
 	 usleep(50000L);
 #endif
 #ifdef sgi
@@ -395,9 +483,8 @@ static int getaudio()
        one of our mute requests made it through anyway. */
 
     if (i <= 0) {
-	hdreq.compression = fHalfDuplex | fHalfDuplexResume;
-	if (sendto(sock, &hdreq, (sizeof(struct soundbuf) - BUFL) +
-		   hdreq.buffer.buffer_len,
+	hdreq.compression = htonl(fHalfDuplex | fHalfDuplexResume);
+	if (sendto(sock, &hdreq, (sizeof(struct soundbuf) - BUFL),
 		   0, (struct sockaddr *) &name, sizeof name) < 0) {
             perror("sending half-duplex resume request");
 	}
@@ -440,15 +527,14 @@ static void freeaudio()
 	struct soundbuf hdreq;
 	struct sockaddr_in name;
 
-	hdreq.compression = fHalfDuplex | fHalfDuplexResume;
+	hdreq.compression = htonl(fHalfDuplex | fHalfDuplexResume);
 	strcpy(hdreq.sendinghost, hostname);
 	hdreq.buffer.buffer_len = 0;
 
 	bcopy((char *) &localhost, (char *) &(name.sin_addr), sizeof localhost);
 	name.sin_family = AF_INET;
 	name.sin_port = htons(Internet_Port);
-	if (sendto(sock, &hdreq, (sizeof(struct soundbuf) - BUFL) +
-		   hdreq.buffer.buffer_len,
+	if (sendto(sock, &hdreq, (sizeof(struct soundbuf) - BUFL),
 		   0, (struct sockaddr *) &name, sizeof name) < 0) {
             perror("sending half-duplex resume request");
 	}
@@ -463,6 +549,8 @@ static void freeaudio()
 
 static void exiting()
 {
+    struct destination *d;
+
 #ifdef PUSH_TO_TALK
     if (rawmode) {
         fprintf(stderr, "\r      \r");
@@ -472,6 +560,22 @@ static void exiting()
 	endwin();
     }
 #endif
+
+#ifdef RTP_SUPPORT
+    for (d = dests; d != NULL; d = d->dnext) {
+	if (d->isrtp) {
+	    char v[1024];
+	    int l;
+
+            l = rtp_make_bye(v, ssrc, "Exiting Speak Freely");
+	    if (sendto(sock, v, l,
+		0, (struct sockaddr *) &(d->ctrl), sizeof d->ctrl) < 0) {
+                perror("sending RTCP BYE packet");
+	    }
+	}
+    }
+#endif
+
     freeaudio();
     exit(0);
 }
@@ -509,11 +613,19 @@ static int sendfile(f)
 		    (compressing ? 2 : 1));
     if (gsmcompress) {
 	sound_packet = compressing ? 3200 : 1600;
-    }
-    if (adpcmcompress) {
+    } else if (adpcmcompress) {
 	sound_packet *= 2;
 	sound_packet -= 4;	      /* Leave room for state at the end */
+    } else if (lpccompress) {
+	sound_packet = 1800;
     }
+
+#ifdef RTP_SUPPORT
+/* 20 millisecond packets is an explicit RTP standard, like it or not. */
+if (rtp) {
+    sound_packet = 160;
+}
+#endif
 #ifdef SHOW_PACKET_SIZE
     printf("Samples per packet = %d\n", sound_packet);
 #endif
@@ -539,10 +651,7 @@ static int sendfile(f)
 	    long startpos;
 
 	    fread(&startpos, sizeof(long), 1, afile);
-#ifdef _M_I86
-	    revlong(&startpos);
-#endif
-	    fseek(afile, startpos, 0);
+	    fseek(afile, ntohl(startpos), 0);
 	} else {
 	    fseek(afile, 0L, 0);
 	}
@@ -551,12 +660,18 @@ static int sendfile(f)
     /* Send a file */
 
     if (afile) {
+#ifdef Solaris
+	struct timeval t1, t2;
+#endif
 #ifdef BSD_like
 	struct timeb t1, t2;
 #endif
 	long et;
 
 	while (
+#ifdef Solaris
+		    gettimeofday(&t1),
+#endif
 #ifdef BSD_like
 		    ftime(&t1),
 #endif
@@ -607,6 +722,12 @@ static int sendfile(f)
 #define kOverhead   3000
 #endif
 
+#ifdef Solaris
+	    gettimeofday(&t2);
+	    et = ((bread * 125) -
+			(t2.tv_sec - t1.tv_sec) * 1000000 +
+			   (t2.tv_usec - t1.tv_usec)) - kOverhead;
+#endif
 #ifdef BSD_like
 	    ftime(&t2);
 	    et = ((bread * 125) -
@@ -619,7 +740,7 @@ static int sendfile(f)
 #endif
 
 	    if (et > 0) {
-#ifdef BSD_like
+#if defined(BSD_like) || defined(Solaris)
 		usleep(et);
 #endif
 #ifdef sgi
@@ -661,6 +782,16 @@ static int sendfile(f)
 
 	/* Send real-time sound. */
 
+#ifdef Solaris
+	setaubufsize(
+#ifdef TINY_PACKETS
+	    sound_packet
+#else
+	    sizeof buf
+#endif
+		    );
+#endif
+
 	if (!getaudio()) {
             fprintf(stderr, "Unable to initialise audio.\n");
 	    return 2;
@@ -705,7 +836,7 @@ static int sendfile(f)
 			freeaudio();
 		    }
 #endif
-                    fprintf(stderr, "\r%s:  ", talking ? "Talk" : "Pause");
+                    fprintf(stderr, talking ? "\rTalk:   " : "\rPause:  ");
 		    fcntl(fileno(stdin), F_SETFL, talking ? O_NDELAY : 0);
 		    if (talking) {
 			/* Discard all backlog sound input. */
@@ -782,11 +913,11 @@ static int sendfile(f)
 		    }
 		    if (debugging) {
                         fprintf(stderr, "Sent %d audio samples in %d bytes.\r\n",
-				osl, netbuf.buffer.buffer_len);
+				osl, soundel);
 		    }
 		}
 	    } else {
-#ifdef BSD_like
+#if defined(BSD_like) || defined(Solaris)
 		usleep(100000L);  /* Wait for some sound to arrive */
 #endif
 #ifdef sgi
@@ -800,7 +931,7 @@ static int sendfile(f)
 
 /*  MAKESESSIONKEY  --	Generate session key with optional start
 			key.  If mode is TRUE, the key will be
-			translated to binary, otherwise it is
+			translated to a string, otherwise it is
 			returned as 16 binary bytes.  */
 
 static void makeSessionKey(key, seed, mode)
@@ -816,13 +947,25 @@ static void makeSessionKey(key, seed, mode)
     if (seed != NULL) {
 	strcat(s, seed);
     }
+
+    /* The following creates a seed for the session key generator
+       based on a collection of volatile and environment-specific
+       information unlikely to be vulnerable (as a whole) to an
+       exhaustive search attack.  If one of these items isn't
+       available on your machine, replace it with something
+       equivalent or, if you like, just delete it. */
+
     sprintf(s + strlen(s), "%u", getpid());
     sprintf(s + strlen(s), "%u", getppid());
     V getcwd(s + strlen(s), 256);
     sprintf(s + strlen(s), "%u", clock());
     V cuserid(s + strlen(s));
     sprintf(s + strlen(s), "%u", time(NULL));
+#ifdef Solaris
+    sysinfo(SI_HW_SERIAL,s + strlen(s), 12);
+#else
     sprintf(s + strlen(s), "%u", gethostid());
+#endif
     getdomainname(s + strlen(s), 256);
     gethostname(s + strlen(s), 256);
     sprintf(s + strlen(s), "%u", getuid());
@@ -861,9 +1004,9 @@ static void makeSessionKey(key, seed, mode)
 static void usage()
 {
     V fprintf(stderr, "%s  --  Speak Freely sound sender.\n", progname);
-    V fprintf(stderr, "            Release 5.3, September 1995.\n");
+    V fprintf(stderr, "            %s.\n", Release);
     V fprintf(stderr, "\n");
-    V fprintf(stderr, "Usage: %s hostname [options] [ file1 / . ]...\n", progname);
+    V fprintf(stderr, "Usage: %s hostname[:port] [options] [ file1 / . ]...\n", progname);
     V fprintf(stderr, "Options: (* indicates defaults)\n");
 #ifdef PUSH_TO_TALK
     V fprintf(stderr, "           -A         Always transmit unless squelched\n");
@@ -877,14 +1020,18 @@ static void usage()
     V fprintf(stderr, "           -Ikey      IDEA encrypt with key\n");
     V fprintf(stderr, "           -Kkey      DES encrypt with key\n");
     V fprintf(stderr, "           -L         Remote loopback\n");
+    V fprintf(stderr, "           -LPC       LPC compression\n");
     V fprintf(stderr, "     *     -M         Manual record gain control\n");
-    V fprintf(stderr, "     *     -N         Do not compress subsequent sound\n");
+    V fprintf(stderr, "           -N         Do not compress subsequent sound\n");
     V fprintf(stderr, "           -Ofile     Use file as key file\n");
-    V fprintf(stderr, "           -Phostname Party line, add host to list\n");
+    V fprintf(stderr, "           -Phostname[:port] Party line, add host to list\n");
     V fprintf(stderr, "     *     -Q         Disable debug output\n");
     V fprintf(stderr, "           -R         Ring--force volume, output to speaker\n");
+#ifdef RTP_SUPPORT
+    V fprintf(stderr, "           -RTP       Use Internet Real-Time Protocol\n");
+#endif
     V fprintf(stderr, "           -Sn        Squelch at level n (0-255)\n");
-    V fprintf(stderr, "           -T         Telephone (GSM) compression\n");
+    V fprintf(stderr, "     *     -T         Telephone (GSM) compression\n");
     V fprintf(stderr, "           -U         Print this message\n");
     V fprintf(stderr, "           -Z\"user..\" Send PGP session key for user(s)\n");
     V fprintf(stderr, "\n");
@@ -911,6 +1058,7 @@ main(argc, argv)
 	hostname[15] = 0;
     }
     gsmh = gsm_create();
+    lpc_init(180);
 
 #ifdef sgi
     usticks = 1000000 / CLK_TCK;
@@ -940,6 +1088,23 @@ main(argc, argv)
 	return 1;
     }
 
+    /* See if there's a face image file for this user.  If
+       so, we'll offer our face in sound packets we send. */
+
+    {   char *cp = getenv("SPEAKFREE_FACE");
+	FILE *facefile;
+
+	if (cp != NULL) {
+            if ((facefile = fopen(cp, "r")) == NULL) {
+                fprintf(stderr, "%s: cannot open SPEAKFREE_FACE file %s\n",
+		    progname, cp);
+	    } else {
+		fclose(facefile);
+		hasFace = TRUE;
+	    }
+	}
+    }
+
     /*	Process command line options.  */
 
     for (i = 1; i < argc; i++) {
@@ -967,6 +1132,12 @@ main(argc, argv)
 #endif
 
                 case 'C':             /* -C  -- Compress sound samples */
+#ifdef RTP_SUPPORT
+		    if (rtp) {
+                        fprintf(stderr, "Simple (-c) compression can't be used in RTP mode.\n");
+			return 2;
+		    }
+#endif
 		    compressing = TRUE;
 		    break;
 
@@ -980,11 +1151,8 @@ main(argc, argv)
 		    return 0;
 
                 case 'F':             /* -F -- ADPCM compression */
-		    if (gsmcompress) {
-                        fprintf(stderr, "Cannot use GSM (-t) and ADPCM (-f) compression simultaneously.\n");
-			return 2;
-		    }
 		    adpcmcompress = TRUE;
+		    gsmcompress = lpccompress = FALSE;
 		    break;
 
                 case 'G':             /* -G  --  Automatic gain control */
@@ -1032,8 +1200,14 @@ main(argc, argv)
 		    }
 		    break;
 
-                case 'L':             /* -L  --  Remote loopback */
-		    loopback = TRUE;
+                case 'L':             /* -L    --  Remote loopback */
+				      /* -LPC  --  LPC compress sound */
+                    if (ucase(op[1]) == 'P') {
+			lpccompress = TRUE;
+			gsmcompress = adpcmcompress = FALSE;
+		    } else {
+			loopback = TRUE;
+		    }
 		    break;
 
                 case 'M':             /* -M  --  Manual record gain control */
@@ -1043,6 +1217,7 @@ main(argc, argv)
                 case 'N':             /* -N  --  Do not compress sound samples */
 		    compressing = FALSE;
 		    gsmcompress = FALSE;
+		    lpccompress = FALSE;
 		    adpcmcompress = FALSE;
 		    break;
 
@@ -1092,8 +1267,34 @@ main(argc, argv)
 		    debugging = FALSE;
 		    break;
 
-                case 'R':             /* -R  --  Ring: divert output to speaker */
-		    ring = TRUE;
+                case 'R':             /* -R    --  Ring: divert output to speaker */
+#ifdef RTP_SUPPORT
+				      /* -RTP  --  Use RTP to transmit */
+                    if (ucase(op[1]) == 'T') {
+			if (compressing) {
+                            fprintf(stderr, "Simple (-c) compression can't be used in RTP mode.\n");
+			    return 2;
+			}
+			rtp = TRUE;
+
+			/* Initialise randomised packet counters. */
+
+			makeSessionKey(md5key, NULL, FALSE);
+			bcopy(md5key, (char *) &ssrc, sizeof ssrc);
+			bcopy(md5key + sizeof ssrc, (char *) &timestamp,
+			      sizeof timestamp);
+			bcopy(md5key + sizeof ssrc + sizeof timestamp,
+			      (char *) &seq, sizeof seq);
+
+			if (sdes == NULL) {
+			    sdesl = rtp_make_sdes(&sdes, ssrc, -1);
+			}
+		    } else {
+#endif
+			ring = TRUE;
+#ifdef RTP_SUPPORT
+		    }
+#endif
 		    break;
 
                 case 'S':             /* -Sn  --  Squelch at level n */
@@ -1105,11 +1306,8 @@ main(argc, argv)
 		    break;
 
                 case 'T':             /* -T -- Telephone (GSM) compression */
-		    if (adpcmcompress) {
-                        fprintf(stderr, "Cannot use GSM (-t) and ADPCM (-f) compression simultaneously.\n");
-			return 2;
-		    }
 		    gsmcompress = TRUE;
+		    lpccompress = adpcmcompress = FALSE;
 		    break;
 
                 case 'U':             /* -U  --  Print usage information */
@@ -1191,8 +1389,9 @@ main(argc, argv)
 				return 2;
 			    }
 			    fread(pgpsb->buffer.buffer_val, (int) flen, 1, kfile);
-			    pgpsb->buffer.buffer_len = flen;
-			    pgpsb->compression = fKeyPGP;
+			    pgpsbl = flen;
+			    pgpsb->buffer.buffer_len = htonl(flen);
+			    pgpsb->compression = htonl(fKeyPGP);
 #ifdef SENDMD5
 			    MD5Init(&md5c);
 			    MD5Update(&md5c, pgpsb->buffer.buffer_val, pgpsb->buffer.buffer_len);
@@ -1229,6 +1428,7 @@ main(argc, argv)
 	}
     }
 
+    exiting();
     gsm_destroy(gsmh);
     desdone();
     return 0;
