@@ -1,6 +1,6 @@
 /*
 
-	Sound interface for Netfone
+	Sound interface for Speak Freely for Unix
 
 	Designed and implemented in July of 1990 by John Walker
 
@@ -8,19 +8,28 @@
 
 #define BUFL	8000
 
-#include "netfone.h"
+#include "speakfree.h"
 
-#ifdef BSD_like
+#if defined(BSD_like) || defined(Solaris)
 
+#ifdef Solaris
+#include <sys/filio.h>
+#else
 #include <sys/dir.h>
 #include <sys/file.h>
+#endif
 
 #include <math.h>
 #include <errno.h>
 
 #include <sys/ioctl.h>
 #ifdef sun
+#include <sys/stropts.h>
+#ifdef Solaris
+#include <sys/audioio.h>
+#else
 #include <sun/audioio.h>
+#endif
 #else /* FreeBSD / Linux */
 #include <machine/soundcard.h>
 #define AUDIO_MIN_GAIN 0
@@ -69,6 +78,20 @@ static unsigned scale_gain(g)
 	((double) g / (double) MAX_GAIN)) + 0.5)));
 }
 
+#ifdef Solaris
+
+/*  SETAUBUFSIZE  --  Preset size of internal /dev/audio buffer segments
+		      Must be called before soundinit()  */
+
+static int aubufsize = 2048;	      /* Default */
+
+void setaubufsize(size)
+  int size;
+{
+    aubufsize = size;
+}
+#endif
+
 
 /*  SOUNDINIT  --  Open the sound peripheral and initialise for
 		   access.  Return TRUE if successful, FALSE
@@ -77,26 +100,51 @@ static unsigned scale_gain(g)
 int soundinit(iomode)
   int iomode;
 {
-    assert(audiof == -1);
-    if ((audiof = open(SoundFile, iomode)) >= 0) {
+    int attempts = 3;
 
-	if ((Audio_fd = open(AUDIO_CTLDEV, O_RDWR)) < 0) {
-	    perror(AUDIO_CTLDEV);
-	    return FALSE;
-	}
-/*fcntl(audiof, F_SETFL, O_NDELAY);*/
+    assert(audiof == -1);
+    while (attempts-- > 0) {
+	if ((audiof = open(SoundFile, iomode)) >= 0) {
+
+	    if ((Audio_fd = open(AUDIO_CTLDEV, O_RDWR)) < 0) {
+		perror(AUDIO_CTLDEV);
+		return FALSE;
+	    }
+    /*fcntl(audiof, F_SETFL, O_NDELAY);*/
 #ifndef sun
-	if (ioctl(audiof, SNDCTL_DSP_NONBLOCK, NULL) < 0) {
-            perror("SNDCTL_DSP_NONBLOCK");
-	    return FALSE;
-	}
-	if (ioctl(audiof, SNDCTL_DSP_GETBLKSIZE, &abuf_size) < 0) {
-            perror("SNDCTL_DSP_GETBLKSIZE");
-	    return FALSE;
-	}
+	    if (ioctl(audiof, SNDCTL_DSP_NONBLOCK, NULL) < 0) {
+                perror("SNDCTL_DSP_NONBLOCK");
+		return FALSE;
+	    }
+	    if (ioctl(audiof, SNDCTL_DSP_GETBLKSIZE, &abuf_size) < 0) {
+                perror("SNDCTL_DSP_GETBLKSIZE");
+		return FALSE;
+	    }
 #else
+	    AUDIO_INITINFO(&Audio_info);
+	    if (iomode != O_RDONLY) {
+		Audio_info.play.sample_rate = 8000;
+		Audio_info.play.channels = 1;
+		Audio_info.play.precision = 8;
+		Audio_info.play.encoding = AUDIO_ENCODING_ULAW;
+	    }
+	    if (iomode != O_WRONLY) {
+		Audio_info.record.sample_rate = 8000;
+		Audio_info.record.channels = 1;
+		Audio_info.record.precision = 8;
+		Audio_info.record.encoding = AUDIO_ENCODING_ULAW;
+#ifdef Solaris
+		Audio_info.record.buffer_size = aubufsize;
 #endif
-	return TRUE;
+	    }
+	    ioctl(Audio_fd, AUDIO_SETINFO, &Audio_info);
+#endif
+	    return TRUE;
+	}
+	if (errno != EINTR) {
+	    break;
+	}
+fprintf(stderr, "Audio open: retrying EINTR attempt %d\n", attempts);
     }
     return FALSE;
 }
@@ -106,9 +154,13 @@ int soundinit(iomode)
 void soundterm()
 {
 	if (audiof >= 0) {
-	   V close(audiof);
-	   V close(Audio_fd);
-	   audiof = -1;
+	    if (close(audiof) < 0) {
+                perror("closing audio device");
+	    }
+	    if (close(Audio_fd) < 0) {
+                perror("closing audio control device");
+	    }
+	    audiof = -1;
 	}
 }
 
@@ -209,7 +261,7 @@ int soundgrab(buf, len)
     int c;
 
 #ifdef sun
-#ifndef BlockingIO
+#ifdef OLDWAY
     if (ioctl(audiof, FIONREAD, &read_size) < 0) {
         perror("soundgrab FIONREAD ioctl failed");
     } else {
@@ -233,10 +285,12 @@ int soundgrab(buf, len)
             perror("soundgrab");
 	}
 	return c;
-#ifndef BlockingIO
+#ifdef sun
+#ifdef OLDWAY
     }
     return 0;
 #endif
+#endif
 }
 
 /*  SOUNDFLUSH	--  Flush any queued sound.  */
@@ -248,8 +302,8 @@ void soundflush()
     int c;
 #endif
 
-    while (TRUE) {
 #ifndef sun
+    while (TRUE) {
 	c = read(audiof, sb, BUFL < abuf_size ? BUFL : abuf_size);
 	if (c < 0 && errno == EAGAIN)
 	    c = 0;
@@ -257,7 +311,12 @@ void soundflush()
             perror("soundflush");
 	if (c <= 0)
 	    break;
+    }
 #else
+#ifdef OLDWAY
+    /* This code is left around just in case some system has
+       trouble with the I_FLUSH ioctl().  */
+    while (TRUE) {
 	long read_size;
 
 #ifndef BlockingIO
@@ -278,8 +337,13 @@ void soundflush()
 	} else {
 	    break;
 	}
-#endif
     }
+#else
+    if (ioctl(audiof, I_FLUSH, FLUSHR)) {
+        perror("soundflush");
+    }
+#endif
+#endif
 }
 #endif
 
@@ -288,7 +352,7 @@ void soundflush()
 /*
 
      Silicon Graphics audio drivers adapted from the stand-alone
-	     Netfone for SGI designed and implemented by:
+	  Speak Freely for SGI designed and implemented by:
 
 		   Paul Schurman (schurman@frs.fi)
 			    Espoo, Finland
